@@ -1,38 +1,29 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
-"""
-    openslides.agenda.views
-    ~~~~~~~~~~~~~~~~~~~~~~~
-
-    Views for the agenda app.
-
-    :copyright: 2011–2013 by the OpenSlides team, see AUTHORS.
-    :license: GNU GPL, see LICENSE for more details.
-"""
 # TODO: Rename all views and template names
 
-from reportlab.platypus import Paragraph
 from datetime import datetime, timedelta
 
-from django.core.urlresolvers import reverse
 from django.contrib import messages
+from django.core.urlresolvers import reverse
 from django.db import transaction
 from django.db.models import Model
-from django.utils.translation import ugettext as _, ugettext_lazy
-from django.views.generic.detail import SingleObjectMixin
+from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext_lazy
+from reportlab.platypus import Paragraph
 
 from openslides.config.api import config
-from openslides.utils.pdf import stylesheet
+from openslides.projector.api import get_active_slide, update_projector
+from openslides.projector.projector import Widget
 from openslides.utils.exceptions import OpenSlidesError
-from openslides.utils.views import (
-    TemplateView, RedirectView, UpdateView, CreateView, DeleteView, PDFView,
-    DetailView, FormView, SingleObjectMixin)
+from openslides.utils.pdf import stylesheet
 from openslides.utils.template import Tab
 from openslides.utils.utils import html_strong
-from openslides.projector.api import get_active_slide, get_slide_from_sid
-from openslides.projector.projector import Widget, SLIDE
+from openslides.utils.views import (CreateView, DeleteView, FormView, PDFView,
+                                    RedirectView, SingleObjectMixin,
+                                    TemplateView, UpdateView)
+
+from .forms import AppendSpeakerForm, ItemForm, ItemOrderForm, RelatedItemForm
 from .models import Item, Speaker
-from .forms import ItemOrderForm, ItemForm, AppendSpeakerForm, RelatedItemForm
 
 
 class Overview(TemplateView):
@@ -66,11 +57,19 @@ class Overview(TemplateView):
         duration = timedelta()
 
         for item in items:
-            if (item.duration is not None and
-                    len(item.duration) > 0):
-                duration_list = item.duration.split(':')
-                duration += timedelta(hours=int(duration_list[0]),
-                                      minutes=int(duration_list[1]))
+            if item.duration is not None and len(item.duration) > 0:
+                if ':' in item.duration:
+                    duration_list = item.duration.split(':')
+                    duration += timedelta(hours=int(duration_list[0]),
+                                          minutes=int(duration_list[1]))
+                else:
+                    hours = int(item.duration) / 60
+                    minutes = int(item.duration) - (60 * hours)
+                    duration += timedelta(hours=hours,
+                                          minutes=minutes)
+                    if minutes < 10:
+                        minutes = "%s%s" % (0, minutes)
+                    item.duration = "%s:%s" % (hours, minutes)
                 if not start is None:
                     item.tooltip = start + duration
 
@@ -83,14 +82,21 @@ class Overview(TemplateView):
             (duration.days * 24 + duration.seconds / 3600.0),
             (duration.seconds / 60.0 % 60))
 
+        active_slide = get_active_slide()
+        if active_slide['callback'] == 'agenda':
+            agenda_is_active = active_slide.get('pk', 'agenda') == 'agenda'
+            active_type = active_slide.get('type', 'text')
+        else:
+            agenda_is_active = None
+            active_type = None
+
         context.update({
             'items': items,
-            'active_sid': get_active_slide(only_sid=True),
+            'agenda_is_active': agenda_is_active,
             'duration': duration,
             'start': start,
             'end': end,
-            'summary': config['presentation_argument'] == 'summary',
-            'show_list': config['presentation_argument'] == 'show_list_of_speakers'})
+            'active_type': active_type})
         return context
 
     @transaction.commit_manually
@@ -99,7 +105,9 @@ class Overview(TemplateView):
             messages.error(
                 request,
                 _('You are not authorized to manage the agenda.'))
+            context = self.get_context_data(**kwargs)
             return self.render_to_response(context)
+
         transaction.commit()
         for item in Item.objects.all():
             form = ItemOrderForm(request.POST, prefix="i%d" % item.id)
@@ -118,7 +126,12 @@ class Overview(TemplateView):
                 break
         else:
             Item.objects.rebuild()
-        # TODO: assure, that it is a valid tree
+            # TODO: assure, that it is a valid tree
+        context = self.get_context_data(**kwargs)
+        transaction.commit()
+
+        if get_active_slide()['callback'] == 'agenda':
+            update_projector()
         context = self.get_context_data(**kwargs)
         transaction.commit()
         return self.render_to_response(context)
@@ -129,20 +142,32 @@ class AgendaItemView(SingleObjectMixin, FormView):
     Show an agenda item.
     """
     # TODO: use 'SingleObjectTemplateResponseMixin' to choose the right template name
-    permission_required = 'agenda.can_see_agenda'
     template_name = 'agenda/view.html'
     model = Item
     context_object_name = 'item'
     form_class = AppendSpeakerForm
 
+    def has_permission(self, request, *args, **kwargs):
+        """
+        Checks if the user has the required permission.
+        """
+        if self.get_object().type == Item.ORGANIZATIONAL_ITEM:
+            permission = request.user.has_perm('agenda.can_see_orga_items')
+        else:
+            permission = request.user.has_perm('agenda.can_see_agenda')
+        return permission
+
     def get_context_data(self, **kwargs):
         self.object = self.get_object()
         list_of_speakers = self.object.get_list_of_speakers()
+        active_slide = get_active_slide()
+        active_type = active_slide.get('type', None)
         kwargs.update({
             'object': self.object,
             'list_of_speakers': list_of_speakers,
-            'is_on_the_list_of_speakers': Speaker.objects.filter(item=self.object, begin_time=None, person=self.request.user).exists(),
-            'show_list': config['presentation_argument'] == 'show_list_of_speakers',
+            'is_on_the_list_of_speakers': Speaker.objects.filter(
+                item=self.object, begin_time=None, person=self.request.user).exists(),
+            'active_type': active_type,
         })
         return super(AgendaItemView, self).get_context_data(**kwargs)
 
@@ -166,16 +191,12 @@ class SetClosed(RedirectView, SingleObjectMixin):
     model = Item
 
     def get_ajax_context(self, **kwargs):
-        context = super(SetClosed, self).get_ajax_context(**kwargs)
-        closed = kwargs['closed']
+        closed = self.kwargs['closed']
         if closed:
-            link = reverse('item_open', args=[self.object.id])
+            link = reverse('item_open', args=[self.object.pk])
         else:
-            link = reverse('item_close', args=[self.object.id])
-        context.update({
-            'closed': kwargs['closed'],
-            'link': link})
-        return context
+            link = reverse('item_close', args=[self.object.pk])
+        return super(SetClosed, self).get_ajax_context(closed=closed, link=link)
 
     def pre_redirect(self, request, *args, **kwargs):
         self.object = self.get_object()
@@ -196,12 +217,14 @@ class ItemUpdate(UpdateView):
     model = Item
     context_object_name = 'item'
     success_url_name = 'item_overview'
+    url_name_args = []
 
     def get_form_class(self):
-        if self.object.related_sid is None:
-            return ItemForm
+        if self.object.content_object:
+            form = RelatedItemForm
         else:
-            return RelatedItemForm
+            form = ItemForm
+        return form
 
 
 class ItemCreate(CreateView):
@@ -214,6 +237,7 @@ class ItemCreate(CreateView):
     context_object_name = 'item'
     form_class = ItemForm
     success_url_name = 'item_overview'
+    url_name_args = []
 
 
 class ItemDelete(DeleteView):
@@ -224,26 +248,74 @@ class ItemDelete(DeleteView):
     model = Item
     question_url_name = 'item_overview'
     success_url_name = 'item_overview'
+    url_name_args = []
 
     def get_answer_options(self):
-        if self.object.children.exists():
-            return [('all', _("Yes, with all child items."))] + self.answer_options
-        else:
-            return self.answer_options
+        """
+        Returns the possible answers to the delete view.
 
-    def pre_post_redirect(self, request, *args, **kwargs):
-        if self.get_answer() == 'all':
-            self.object.delete(with_children=True)
-            messages.success(
-                request,
-                _("Item %s and his children were successfully deleted.")
-                % html_strong(self.object))
-        elif self.get_answer() == 'yes':
-            self.object.delete(with_children=False)
-            messages.success(
-                request,
-                _("Item %s was successfully deleted.")
-                % html_strong(self.object))
+        'all' is a possible answer, when the item has children.
+        """
+        # Cache the result in the request, so when the children are deleted, the
+        # result does not change
+        try:
+            options = self.item_delete_answer_options
+        except AttributeError:
+            if self.object.children.exists():
+                options = [('all', _("Yes, with all child items."))] + self.answer_options
+            else:
+                options = self.answer_options
+            self.item_delete_answer_options = options
+        return options
+
+    def on_clicked_yes(self):
+        """
+        Deletes the item but not its children.
+        """
+        self.object.delete(with_children=False)
+
+    def on_clicked_all(self):
+        """
+        Deletes the item and its children.
+        """
+        self.object.delete(with_children=True)
+
+    def get_final_message(self):
+        """
+        Prints the success message to the user.
+        """
+        # OpenSlidesError (invalid answer) should never be raised here because
+        # this method should only be called if the answer is 'yes' or 'all'.
+        if self.get_answer() == 'yes':
+            message = _('Item %s was successfully deleted.') % html_strong(self.object)
+        else:
+            message = _('Item %s and its children were successfully deleted.') % html_strong(self.object)
+        return message
+
+
+class CreateRelatedAgendaItemView(SingleObjectMixin, RedirectView):
+    """
+    View to create and agenda item for a related object.
+
+    This view is only for subclassing in views of related apps. You
+    have to define 'model = ....'
+    """
+    permission_required = 'agenda.can_manage_agenda'
+    url_name = 'item_overview'
+    url_name_args = []
+
+    def get(self, request, *args, **kwargs):
+        """
+        Set self.object to the relevant object.
+        """
+        self.object = self.get_object()
+        return super(CreateRelatedAgendaItemView, self).get(request, *args, **kwargs)
+
+    def pre_redirect(self, request, *args, **kwargs):
+        """
+        Create the agenda item.
+        """
+        self.item = Item.objects.create(content_object=self.object)
 
 
 class AgendaPDF(PDFView):
@@ -283,6 +355,8 @@ class SpeakerAppendView(SingleObjectMixin, RedirectView):
                 Speaker.objects.add(item=self.object, person=request.user)
             except OpenSlidesError, e:
                 messages.error(request, e)
+            else:
+                messages.success(request, _('You were successfully added to the list of speakers.'))
 
 
 class SpeakerDeleteView(DeleteView):
@@ -460,11 +534,14 @@ class CurrentListOfSpeakersView(RedirectView):
         """
         Returns the current Item, or None, if the current Slide is not an Agenda Item.
         """
-        slide = get_slide_from_sid(get_active_slide(only_sid=True), element=True)
-        if not isinstance(slide, Item):
-            return None
+        active_slide = get_active_slide()
+        if active_slide['callback'] == 'agenda':
+            try:
+                return Item.objects.get(pk=active_slide.get('pk', None))
+            except Item.DoesNotExist:
+                return None
         else:
-            return slide
+            return None
 
     def get_redirect_url(self):
         """
@@ -505,6 +582,8 @@ class CurrentListOfSpeakersView(RedirectView):
                         Speaker.objects.add(self.request.user, item)
                     except OpenSlidesError, e:
                         messages.error(request, e)
+                    else:
+                        messages.success(request, _('You were successfully added to the list of speakers.'))
                     finally:
                         reverse_to_dashboard = False
                 else:
@@ -533,10 +612,16 @@ class CurrentListOfSpeakersView(RedirectView):
                 messages.success(request, _('%s is now finished.') % current_speaker)
             reverse_to_dashboard = True
 
-        if reverse_to_dashboard or not self.request.user.has_perm('agenda.can_see_agenda'):
-            return reverse('dashboard')
+        if item.type == Item.ORGANIZATIONAL_ITEM:
+            if reverse_to_dashboard or not self.request.user.has_perm('agenda.can_see_orga_items'):
+                return reverse('dashboard')
+            else:
+                return reverse('item_view', args=[item.pk])
         else:
-            return reverse('item_view', args=[item.pk])
+            if reverse_to_dashboard or not self.request.user.has_perm('agenda.can_see_agenda'):
+                return reverse('dashboard')
+            else:
+                return reverse('item_view', args=[item.pk])
 
 
 def register_tab(request):
@@ -557,6 +642,14 @@ def get_widgets(request):
     """
     Returns the agenda widget for the projector tab.
     """
+    active_slide = get_active_slide()
+    if active_slide['callback'] == 'agenda':
+        agenda_is_active = active_slide.get('pk', 'agenda') == 'agenda'
+        active_type = active_slide.get('type', 'text')
+    else:
+        agenda_is_active = None
+        active_type = None
+
     return [
         Widget(
             request,
@@ -564,15 +657,18 @@ def get_widgets(request):
             display_name=_('Agenda'),
             template='agenda/widget.html',
             context={
-                'agenda': SLIDE['agenda'],
+                'agenda_is_active': agenda_is_active,
                 'items': Item.objects.all(),
-                'summary': config['presentation_argument'] == 'summary',
-                'speakers': config['presentation_argument'] == 'show_list_of_speakers'},
-            permission_required='projector.can_manage_projector'),
+                'active_type': active_type},
+            permission_required='projector.can_manage_projector',
+            default_column=1,
+            default_weight=20),
 
         Widget(
             request,
             name='append_to_list_of_speakers',
             display_name=_('List of speakers'),
             template='agenda/speaker_widget.html',
-            permission_required='agenda.can_be_speaker')]
+            permission_required='agenda.can_be_speaker',
+            default_column=1,
+            default_weight=30)]
