@@ -1,19 +1,10 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
-"""
-    openslides.global_settings
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-    OpenSlides default settings.
-
-    :copyright: 2011–2013 by OpenSlides team, see AUTHORS.
-    :license: GNU GPL, see LICENSE for more details.
-"""
 
 import os
-import sys
 
-from openslides.main import fs2unicode
+from django.utils.translation import ugettext_lazy
+
+from openslides.utils.main import filesystem2unicode
 
 
 SITE_ROOT = os.path.realpath(os.path.dirname(__file__))
@@ -27,13 +18,12 @@ LOGIN_REDIRECT_URL = '/'
 
 SESSION_COOKIE_NAME = 'OpenSlidesSessionID'
 
-ugettext = lambda s: s
-
 LANGUAGES = (
-    ('de', ugettext('German')),
-    ('en', ugettext('English')),
-    ('fr', ugettext('French')),
-    ('cs', ugettext('Czech')),
+    ('cs', ugettext_lazy('Czech')),
+    ('en', ugettext_lazy('English')),
+    ('fr', ugettext_lazy('French')),
+    ('de', ugettext_lazy('German')),
+    ('pt', ugettext_lazy('Portuguese')),
 )
 
 # If you set this to False, Django will make some optimizations so as not
@@ -45,7 +35,7 @@ USE_I18N = True
 USE_L10N = True
 
 LOCALE_PATHS = (
-    fs2unicode(os.path.join(SITE_ROOT, 'locale')),
+    filesystem2unicode(os.path.join(SITE_ROOT, 'locale')),
 )
 
 # URL that handles the media served from MEDIA_ROOT. Make sure to use a
@@ -55,7 +45,7 @@ MEDIA_URL = '/media/'
 
 # Absolute path to the directory that holds static media from ``collectstatic``
 # Example: "/home/media/static.lawrence.com/"
-STATIC_ROOT = fs2unicode(os.path.join(SITE_ROOT, '../collected-site-static'))
+STATIC_ROOT = filesystem2unicode(os.path.join(SITE_ROOT, '../collected-site-static'))
 
 # URL that handles the media served from STATIC_ROOT. Make sure to use a
 # trailing slash if there is a path component (optional in other cases).
@@ -65,18 +55,14 @@ STATIC_URL = '/static/'
 # Additional directories containing static files (not application specific)
 # Examples: "/home/media/lawrence.com/extra-static/"
 STATICFILES_DIRS = (
-    fs2unicode(os.path.join(SITE_ROOT, 'static')),
+    filesystem2unicode(os.path.join(SITE_ROOT, 'static')),
 )
 
-#XXX: Note this setting (as well as our workaround finder)
-#     can be removed again once django-bug-#18404 has been resolved
 STATICFILES_FINDERS = (
     'django.contrib.staticfiles.finders.FileSystemFinder',
-    'openslides.utils.staticfiles.AppDirectoriesFinder',
+    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
 )
 
-MESSAGE_STORAGE = 'django.contrib.messages.storage.cookie.CookieStorage'
-
 # List of callables that know how to import templates from various sources.
 TEMPLATE_LOADERS = (
     'django.template.loaders.filesystem.Loader',
@@ -100,7 +86,7 @@ TEMPLATE_DIRS = (
     # "C:/www/django/templates".
     # Always use forward slashes, even on Windows.
     # Don't forget to use absolute paths, not relative paths.
-    fs2unicode(os.path.join(SITE_ROOT, 'templates')),
+    filesystem2unicode(os.path.join(SITE_ROOT, 'templates')),
 )
 
 INSTALLED_APPS = (
@@ -111,7 +97,7 @@ INSTALLED_APPS = (
     'django.contrib.staticfiles',
     'django.contrib.humanize',
     'mptt',
-    'openslides.utils',
+    'haystack',  # full-text-search
     'openslides.poll',
     'openslides.core',
     'openslides.account',
@@ -140,9 +126,16 @@ CACHES = {
     }
 }
 
-TEST_RUNNER = 'discover_runner.DiscoverRunner'
-TEST_DISCOVER_TOP_LEVEL = os.path.dirname(os.path.dirname(__file__))
-
 # Hosts/domain names that are valid for this site; required if DEBUG is False
 # See https://docs.djangoproject.com/en/1.5/ref/settings/#allowed-hosts
 ALLOWED_HOSTS = ['*']
+
+# Use Haystack with Whoosh for full text search
+HAYSTACK_CONNECTIONS = {
+    'default': {
+        'ENGINE': 'haystack.backends.whoosh_backend.WhooshEngine'
+    },
+}
+
+# Haystack updates search index after each save/delete action by apps
+HAYSTACK_SIGNAL_PROCESSOR = 'haystack.signals.RealtimeSignalProcessor'
