@@ -1,47 +1,32 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
-"""
-    openslides.motion.views
-    ~~~~~~~~~~~~~~~~~~~~~~~
 
-    Views for the motion app.
-
-    The views are automaticly imported from openslides.motion.urls.
-
-    :copyright: (c) 2011–2013 by the OpenSlides team, see AUTHORS.
-    :license: GNU GPL, see LICENSE for more details.
-"""
-
-from django.core.urlresolvers import reverse
 from django.contrib import messages
-from django.db import transaction
-from django.db.models import Model
-from django.utils.text import slugify
-from django.utils.translation import ugettext as _, ugettext_lazy, ugettext_noop
-from django.views.generic.detail import SingleObjectMixin
+from django.core.urlresolvers import reverse
 from django.http import Http404, HttpResponseRedirect
-
+from django.utils.text import slugify
+from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext_lazy, ugettext_noop
 from reportlab.platypus import SimpleDocTemplate
 
-from openslides.utils.pdf import stylesheet
-from openslides.utils.views import (
-    TemplateView, RedirectView, UpdateView, CreateView, DeleteView, PDFView,
-    DetailView, ListView, FormView, QuestionMixin, SingleObjectMixin)
+from openslides.agenda.views import CreateRelatedAgendaItemView as _CreateRelatedAgendaItemView
+from openslides.config.api import config
+from openslides.poll.views import PollFormView
+from openslides.projector.api import get_active_slide, update_projector
+from openslides.projector.projector import Widget
 from openslides.utils.template import Tab
 from openslides.utils.utils import html_strong, htmldiff
-from openslides.poll.views import PollFormView
-from openslides.projector.api import get_active_slide
-from openslides.projector.projector import Widget, SLIDE
-from openslides.config.api import config
-from openslides.agenda.models import Item
+from openslides.utils.views import (CreateView, DeleteView, DetailView,
+                                    FormView, ListView, PDFView, QuestionView,
+                                    RedirectView, SingleObjectMixin, UpdateView)
 
-from .models import (Motion, MotionSubmitter, MotionSupporter, MotionPoll,
-                     MotionVersion, State, WorkflowError, Category)
-from .forms import (BaseMotionForm, MotionSubmitterMixin, MotionSupporterMixin,
-                    MotionDisableVersioningMixin, MotionCategoryMixin,
-                    MotionIdentifierMixin, MotionWorkflowMixin, MotionImportForm)
-from .pdf import motions_to_pdf, motion_to_pdf, motion_poll_to_pdf
 from .csv_import import import_motions
+from .forms import (BaseMotionForm, MotionCategoryMixin,
+                    MotionDisableVersioningMixin, MotionIdentifierMixin,
+                    MotionImportForm, MotionSubmitterMixin,
+                    MotionSupporterMixin, MotionWorkflowMixin)
+from .models import (Category, Motion, MotionPoll, MotionSubmitter,
+                     MotionSupporter, MotionVersion, State)
+from .pdf import motion_poll_to_pdf, motion_to_pdf, motions_to_pdf
 
 
 class MotionListView(ListView):
@@ -128,6 +113,20 @@ class MotionEditMixin(object):
             MotionSupporter.objects.bulk_create(
                 [MotionSupporter(motion=self.object, person=person)
                  for person in form.cleaned_data['supporter']])
+
+        # Save the attachments
+        self.object.attachments.clear()
+        self.object.attachments.add(*form.cleaned_data['attachments'])
+
+        # Update the projector if the motion is on it. This can not be done in
+        # the model, because bulk_create does not call the save method.
+        active_slide = get_active_slide()
+        active_slide_pk = active_slide.get('pk', None)
+        if (active_slide['callback'] == 'motion' and
+                unicode(self.object.pk) == unicode(active_slide_pk)):
+            update_projector()
+
+        messages.success(self.request, self.get_success_message())
         return HttpResponseRedirect(self.get_success_url())
 
     def get_form_class(self):
@@ -272,7 +271,7 @@ class MotionUpdateView(MotionEditMixin, UpdateView):
             self.version = self.object.get_last_version()
             self.used_new_version = False
 
-motion_edit = MotionUpdateView.as_view()
+motion_update = MotionUpdateView.as_view()
 
 
 class MotionDeleteView(DeleteView):
@@ -288,7 +287,7 @@ class MotionDeleteView(DeleteView):
         """
         return self.get_object().get_allowed_actions(request.user)['delete']
 
-    def get_success_message(self):
+    def get_final_message(self):
         return _('%s was successfully deleted.') % _('Motion')
 
 motion_delete = MotionDeleteView.as_view()
@@ -303,26 +302,34 @@ class VersionDeleteView(DeleteView):
     success_url_name = 'motion_detail'
 
     def get_object(self):
-        motion_id = int(self.kwargs.get('pk'))
-        version_number = int(self.kwargs.get('version_number'))
-        return MotionVersion.objects.get(motion=motion_id,
-                                         version_number=version_number)
+        try:
+            motion = Motion.objects.get(pk=int(self.kwargs.get('pk')))
+        except Motion.DoesNotExist:
+            raise Http404('Motion %s not found.' % self.kwargs.get('pk'))
+        try:
+            version = MotionVersion.objects.get(
+                motion=motion,
+                version_number=int(self.kwargs.get('version_number')))
+        except MotionVersion.DoesNotExist:
+            raise Http404('Version %s not found.' % self.kwargs.get('version_number'))
+        if version == motion.active_version:
+            raise Http404('You can not delete the active version of a motion.')
+        return version
 
-    def get_success_url_name_args(self):
+    def get_url_name_args(self):
         return (self.object.motion_id, )
 
 version_delete = VersionDeleteView.as_view()
 
 
-class VersionPermitView(SingleObjectMixin, QuestionMixin, RedirectView):
+class VersionPermitView(SingleObjectMixin, QuestionView):
     """
     View to permit a version of a motion.
     """
     model = Motion
-    question_url_name = 'motion_version_detail'
-    success_url_name = 'motion_version_detail'
-    success_message = ugettext_lazy('Version successfully permitted.')
+    final_message = ugettext_lazy('Version successfully permitted.')
     permission_required = 'motion.can_manage_motion'
+    question_url_name = 'motion_version_detail'
 
     def get(self, *args, **kwargs):
         """
@@ -333,7 +340,7 @@ class VersionPermitView(SingleObjectMixin, QuestionMixin, RedirectView):
         try:
             self.version = self.object.versions.get(version_number=int(version_number))
         except MotionVersion.DoesNotExist:
-            raise Http404('Version %s not found' % version_number)
+            raise Http404('Version %s not found.' % version_number)
         return super(VersionPermitView, self).get(*args, **kwargs)
 
     def get_url_name_args(self):
@@ -342,13 +349,13 @@ class VersionPermitView(SingleObjectMixin, QuestionMixin, RedirectView):
         """
         return [self.object.pk, self.version.version_number]
 
-    def get_question(self):
+    def get_question_message(self):
         """
         Return a string, shown to the user as question to permit the version.
         """
         return _('Are you sure you want permit version %s?') % self.version.version_number
 
-    def case_yes(self):
+    def on_clicked_yes(self):
         """
         Activate the version, if the user chooses 'yes'.
         """
@@ -400,7 +407,7 @@ class VersionDiffView(DetailView):
 version_diff = VersionDiffView.as_view()
 
 
-class SupportView(SingleObjectMixin, QuestionMixin, RedirectView):
+class SupportView(SingleObjectMixin, QuestionView):
     """
     View to support or unsupport a motion.
 
@@ -434,7 +441,7 @@ class SupportView(SingleObjectMixin, QuestionMixin, RedirectView):
         else:
             return True
 
-    def get_question(self):
+    def get_question_message(self):
         """
         Return the question string.
         """
@@ -443,7 +450,7 @@ class SupportView(SingleObjectMixin, QuestionMixin, RedirectView):
         else:
             return _('Do you really want to unsupport this motion?')
 
-    def case_yes(self):
+    def on_clicked_yes(self):
         """
         Append or remove the request.user from the motion.
 
@@ -459,7 +466,7 @@ class SupportView(SingleObjectMixin, QuestionMixin, RedirectView):
                 self.object.unsupport(person=user)
                 self.object.write_log([ugettext_noop('Motion unsupported')], user)
 
-    def get_success_message(self):
+    def get_final_message(self):
         """
         Return the success message.
         """
@@ -468,12 +475,6 @@ class SupportView(SingleObjectMixin, QuestionMixin, RedirectView):
         else:
             return _("You have unsupported this motion successfully.")
 
-    def get_redirect_url(self, **kwargs):
-        """
-        Return the url, the view should redirect to.
-        """
-        return self.object.get_absolute_url()
-
 motion_support = SupportView.as_view(support=True)
 motion_unsupport = SupportView.as_view(support=False)
 
@@ -484,6 +485,7 @@ class PollCreateView(SingleObjectMixin, RedirectView):
     """
     permission_required = 'motion.can_manage_motion'
     model = Motion
+    url_name = 'motion_poll_detail'
 
     def get(self, request, *args, **kwargs):
         """
@@ -502,9 +504,9 @@ class PollCreateView(SingleObjectMixin, RedirectView):
 
     def get_redirect_url(self, **kwargs):
         """
-        Return the URL to the EditView of the poll.
+        Return the URL to the UpdateView of the poll.
         """
-        return reverse('motion_poll_edit', args=[self.object.pk, self.poll.poll_number])
+        return reverse('motion_poll_update', args=[self.object.pk, self.poll.poll_number])
 
 poll_create = PollCreateView.as_view()
 
@@ -567,7 +569,7 @@ class PollUpdateView(PollMixin, PollFormView):
         self.object.write_log([ugettext_noop('Poll updated')], self.request.user)
         return value
 
-poll_edit = PollUpdateView.as_view()
+poll_update = PollUpdateView.as_view()
 
 
 class PollDeleteView(PollMixin, DeleteView):
@@ -577,11 +579,11 @@ class PollDeleteView(PollMixin, DeleteView):
 
     model = MotionPoll
 
-    def case_yes(self):
+    def on_clicked_yes(self):
         """
         Write a log message, if the form is valid.
         """
-        super(PollDeleteView, self).case_yes()
+        super(PollDeleteView, self).on_clicked_yes()
         self.object.motion.write_log([ugettext_noop('Poll deleted')], self.request.user)
 
     def get_redirect_url(self, **kwargs):
@@ -645,17 +647,21 @@ class MotionSetStateView(SingleObjectMixin, RedirectView):
         Save the new state and write a log message.
         """
         self.object = self.get_object()
-        try:
-            if self.reset:
-                self.object.reset_state()
-            else:
-                self.object.set_state(int(kwargs['state']))
-        except WorkflowError, e:  # TODO: Is a WorkflowError still possible here?
-            messages.error(request, e)
+        success = False
+        if self.reset:
+            self.object.reset_state()
+            success = True
+        elif self.object.state.id == int(kwargs['state']):
+            messages.error(request, _('You can not set the state of the motion. It is already done.'))
+        elif int(kwargs['state']) not in [state.id for state in self.object.state.next_states.all()]:
+            messages.error(request, _('You can not set the state of the motion to %s.') % _(State.objects.get(pk=int(kwargs['state'])).name))
         else:
+            self.object.set_state(int(kwargs['state']))
+            success = True
+        if success:
             self.object.save(update_fields=['state', 'identifier'])
             self.object.write_log(
-                message_list=[ugettext_noop('State changed to'), ' %s' % self.object.state.name],
+                message_list=[ugettext_noop('State changed to'), ' ', self.object.state.name],  # TODO: Change string to 'State set to ...'
                 person=self.request.user)
             messages.success(request,
                              _('The state of the motion was set to %s.')
@@ -665,30 +671,20 @@ set_state = MotionSetStateView.as_view()
 reset_state = MotionSetStateView.as_view(reset=True)
 
 
-class CreateAgendaItemView(SingleObjectMixin, RedirectView):
+class CreateRelatedAgendaItemView(_CreateRelatedAgendaItemView):
     """
     View to create and agenda item for a motion.
     """
-    permission_required = 'agenda.can_manage_agenda'
     model = Motion
-    url_name = 'item_overview'
-    url_name_args = []
-
-    def get(self, request, *args, **kwargs):
-        """
-        Set self.object to a motion.
-        """
-        self.object = self.get_object()
-        return super(CreateAgendaItemView, self).get(request, *args, **kwargs)
 
     def pre_redirect(self, request, *args, **kwargs):
         """
         Create the agenda item.
         """
-        self.item = Item.objects.create(related_sid=self.object.sid)
+        super(CreateRelatedAgendaItemView, self).pre_redirect(request, *args, **kwargs)
         self.object.write_log([ugettext_noop('Agenda item created')], self.request.user)
 
-create_agenda_item = CreateAgendaItemView.as_view()
+create_agenda_item = CreateRelatedAgendaItemView.as_view()
 
 
 class MotionPDFView(SingleObjectMixin, PDFView):
@@ -751,6 +747,7 @@ class CategoryCreateView(CreateView):
     permission_required = 'motion.can_manage_motion'
     model = Category
     success_url_name = 'motion_category_list'
+    url_name_args = []
 
 category_create = CategoryCreateView.as_view()
 
@@ -759,6 +756,7 @@ class CategoryUpdateView(UpdateView):
     permission_required = 'motion.can_manage_motion'
     model = Category
     success_url_name = 'motion_category_list'
+    url_name_args = []
 
 category_update = CategoryUpdateView.as_view()
 
@@ -767,6 +765,7 @@ class CategoryDeleteView(DeleteView):
     permission_required = 'motion.can_manage_motion'
     model = Category
     question_url_name = 'motion_category_list'
+    url_name_args = []
     success_url_name = 'motion_category_list'
 
 category_delete = CategoryDeleteView.as_view()
@@ -839,4 +838,6 @@ def get_widgets(request):
         display_name=_('Motions'),
         template='motion/widget.html',
         context={'motions': Motion.objects.all()},
-        permission_required='projector.can_manage_projector')]
+        permission_required='projector.can_manage_projector',
+        default_column=1,
+        default_weight=40)]
