@@ -1,10 +1,3 @@
-/**
- * OpenSlides utils functions
- *
- * :copyright: 2011, 2012 by OpenSlides team, see AUTHORS.
- * :license: GNU GPL, see LICENSE for more details.
- */
-
 $(function () {
     $('.button').click(function (event) {
         button = $(this);
@@ -46,6 +39,10 @@ $(function () {
                 }
                 link.addClass('btn-primary');
                 link.children('i').addClass('icon-white');
+                // set page_num to 1 if a pdf is activated
+                if ( link.hasClass('choose-pdf') ) {
+                    $('#page_num').val(1);
+                }
             }
         });
     });
